<?php
/*
Plugin Name: ChCounter Widget
Plugin URI: http://wordpress.org/extend/plugins/chcounter-widget/
Description: Integrate chCounter into Wordpress as widget.
Version: 3.3.1
Author: Kolja Schleich

Copyright 2007-2016  Kolja Schleich  (email : kolja [dot] schleich [at] googlemail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class chCounterWidget extends WP_Widget
{
	/**
	 * Plugin Version
	 *
	 * @var string
	 */
	var $version = '3.3.1';
	
	
	/**
	 * url to the plugin
	 *
	 * @var string
	 */
	var $plugin_url;

	
	/**
	 * path to the plugin
	 *
	 * @var string
	 */
	var $plugin_path;
	
	
	/**
	 * Class constructor
	 *
	 * @param none
	 * @return void
	 */
	function __construct( $init = false )
	{
		// define constants
		if ( !defined( 'WP_CONTENT_URL' ) )
			define( 'WP_CONTENT_URL', get_option( 'siteurl' ) . '/wp-content' );
		if ( !defined( 'WP_PLUGIN_URL' ) )
			define( 'WP_PLUGIN_URL', WP_CONTENT_URL. '/plugins' );
			
		// define plugin url and path
		$this->plugin_url = esc_url(WP_PLUGIN_URL.'/'.basename(__FILE__, '.php'));
		$this->plugin_path = dirname( __FILE__ );
		
		if ( $init ) {
			define( 'CHCOUNTER_WIDGET_VERSION', $this->version );
			define( 'CHCOUNTER_WIDGET_PATH', $this->plugin_path );
			define( 'CHCOUNTER_WIDGET_URL', $this->plugin_url );
			
			// register activation function
			register_activation_hook(__FILE__, array(&$this, 'activate') );
			// register uninstallation function
			register_uninstall_hook(__FILE__, array('chCounterWidget', 'uninstall'));
			
			// load plugin translations
			load_plugin_textdomain( 'chcounter', false, basename(__FILE__, '.php').'/languages' );
		
			// register widget
			//add_action( 'widgets_init', array(&$this, 'register') );

			// add stylesheet before theme styles
			add_action( 'wp_enqueue_scripts', array(&$this, 'addStyles'), 1 );
			
			// add admin menu
			add_action( 'admin_menu', array(&$this, 'addAdminMenu') );
		} else {
			$widget_ops = array('classname' => 'chcounter_widget', 'description' => __('Include chCounter', 'sponsors-slideshow') );
			parent::__construct('chcounter', __('chCounter', 'chcounter'), $widget_ops);
		} 
	}
	function chCounterWidget()
	{
		$this->__construct();
	}
	
	
	/**
	 * get version
	 *
	 * @param nine
	 * @return string
	 */
	function getVersion()
	{
		return $this->version;
	}
	
	
	/**
	 * get available parameters to display
	 *
	 * @param none
	 * @return array of parameters
	 */
	function getParameters()
	{
		$params = array();
		$params["total"] = array( "label" => __('Total Visitors', 'chcounter'), "value" => "{V_TOTAL_VISITORS}" );
		$params["today"] = array("label" => __('Visitors today', 'chcounter'), "value" => "{V_VISITORS_TODAY}" );
		$params["yesterday"] = array( "label" => __('Visitors yesterday', 'chcounter'), "value" => "{V_VISITORS_YESTERDAY}" );
		$params["perday"] = array( "label" => __('Visitors per day', 'chcounter'), "value" => "{V_VISITORS_PER_DAY}" );
		$params["maxperday"] = array( "label" => __('Max. visitors per day', 'chcounter'), "value" => "{V_MAX_VISITORS_PER_DAY}" );
		$params["maxperdaydate"] = array( "label" => __('Max. visitors per day date', 'chcounter'), "value" => "{V_MAX_VISITORS_PER_DAY_DATE}" );
		$params["online"] = array( "label" => __('Curently online', 'chcounter'), "value" => "{V_VISITORS_CURRENTLY_ONLINE}" );
		$params["maxonline"] = array( "label" => __('Max. online', 'chcounter'), "value" => "{V_MAX_VISITORS_ONLINE}" );
		$params["maxonlinedate"] = array( "label" => __('Max. online date', 'chcounter'), "value" => "{V_MAX_VISITORS_ONLINE_DATE}" );
		$params["totalpageviews"] = array( "label" => __('Total page views', 'chcounter'), "value" => "{V_TOTAL_PAGE_VIEWS}" );
		$params["pageviewstoday"] = array( "label" => __('Page views today', 'chcounter'), "value" => "{V_PAGE_VIEWS_TODAY}" );
		$params["pageviewsyesterday"] = array( "label" => __('Page views yesterday', 'chcounter'), "value" => "{V_PAGE_VIEWS_YESTERDAY}" );
		$params["pageviewsperday"] = array( "label" => __('Page views per day', 'chcounter'), "value" => "{V_PAGE_VIEWS_PER_DAY}" );
		$params["maxpageviewsperday"] = array( "label" => __('Max. page views per day', 'chcounter'), "value" => "{V_MAX_PAGE_VIEWS_PER_DAY}" );
		$params["maxpageviewsperdaydate"] = array( "label" => __('Max. page views per day date', 'chcounter'), "value" => "{V_MAX_PAGE_VIEWS_PER_DAY_DATE}" );
		$params["pageviewsthispage"] = array( "label" => __('Page views of current page', 'chcounter'), "value" => "{V_PAGE_VIEWS_THIS_PAGE}" );
		$params["pageviewscurrentvisitor"] = array( "label" => __('Page views of current visitor', 'chcounter'), "value" => "{V_PAGE_VIEWS_OF_CURRENT_VISITOR}" );
		$params["pageviewspervisitor"] = array( "label" => __('Page views per visitor', 'chcounter'), "value" => "{V_PAGE_VIEWS_PER_VISITOR}" );
		$params["javascriptactivated"] = array( "label" => __('Javascript activated', 'chcounter'), "value" => "{V_JS_PERCENTAGE}" );
		$params["counterstart"] = array( "label" => __('Counterstart', 'chcounter'), "value" => "{V_COUNTER_START}" );
		$params["stats"] = array( "label" => __('Statistics', 'chcounter'), "value" => "{V_COUNTER_URL}" );

		return $params;
	}

	
	/**
	 * display chCounter Widget
	 *
	 * Usually this function is invoked by the Wordpress widget system.
	 * However it can also be called manually via chcounter_widget_display().
	 *
	 * @param array/string $args
	 * @return void
	 */
	function widget($args, $instance)
	{
		if ( is_string($args) )
			$args = array( 'widget_title' => $args );
			
		$options = get_option( 'chcounter_widget' );
		$params = $this->getParameters();

		$defaults = array(
			'before_widget' => '<li id="chcounter" class="widget '.get_class($this).'_'.__FUNCTION__.'">',
			'after_widget' => '</li>',
			'before_title' => '<h2 class="widgettitle">',
			'after_title' => '</h2>',
			'widget_title' => isset($instance['title']) ? htmlspecialchars($instance['title']) : '',
		);
		
		$args = array_merge( $defaults, $args );
		extract( $args );
		
		$file = $this->plugin_path.'/chcounter/counter.php';
		if ( file_exists($file)) {
			if ( isset($instance['invisible']) && $instance['invisible'] == 1 ) {
				$chCounter_visible = 0;
				include_once($file);
			} else {
				echo $before_widget;
				if ( !empty($widget_title) ) echo $before_title . stripslashes($widget_title) . $after_title;

				$counter_template = '';
				
				if ( count($options['params']['active']) > 0 ) {
					foreach ( $options['params']['active'] AS $order => $param ) {
						if ( 'stats' == $param )
							$counter_template .= "<li id='chcounter_stats'><span class='icon'><a target='_blank' href='".$params['stats']['value']."/stats/index.php'><img src='".$params['stats']['value']."/images/stats.png' alt='".$params['stats']['label']."' title='".$params['stats']['label']."' /></a></span><span class='text'><a target='_blank' href='".$params['stats']['value']."/stats/index.php'>".$params['stats']['label']."</a></span></li>";
						else
							$counter_template .= "<li>".$params[$param]['label'].": ".$params[$param]['value']."</li>";
					}
				}
				
				$chCounter_template = <<<TEMPLATE
								 <ul>$counter_template</ul>
TEMPLATE;
				include_once($file);
				echo $after_widget;
			}
		} else {
			echo $before_widget . $before_title . __( 'chCounter Error', 'chcounter' ) .$after_title.__( 'Could not find the chcounter installation. Please check your settings.', 'chcounter' ).$after_widget;
		}
			
		return;
	}


	/**
	 * displays admin page 
	 *
	 * @param none
	 * @return void
	 */
	function displayAdminPage()
	{
		$params = $this->getParameters();
		
		$chcounter_install_dir = $this->plugin_path."/chcounter/install";
		if (isset($_GET['delete_install_dir'])) {
			$this->removeDir($chcounter_install_dir);
			echo "<div id='message' class='updated fade'><p>".__('chCounter installation directory deleted', 'chcounter')."</p></div>";
		}
		if (file_exists($chcounter_install_dir)) {
			if ($this->chCounterIsInstalled()) {
				echo "<div class='updated fade error'><p>".sprintf(__('The chCounter installation directory exists, but chCounter seems to be already installed. If this is not the case you should <a href="%s" target="_blank">install</a> chCounter. Otherwise, you should <a href="%s">delete</a> the installation directory to prevent misuse.', 'chcounter'), $this->plugin_url.'/chcounter/install/install.php', '?page=chcounter-widget.php&delete_install_dir')."</p></div>";
			} else {
				echo "<div class='updated fade error'><p>".sprintf(__('The chCounter installation directory exists. If you visit this page for the first time you have to <a href="%s" target="_blank">install</a> chCounter. After installation <a href="%s">delete</a> the installation directory to prevent misuse.', 'chcounter'), $this->plugin_url.'/chcounter/install/install.php', '?page=chcounter-widget.php&delete_install_dir')."</p></div>";
			}
		}

		if ( isset($_POST['updateSettings']) ) {
			check_admin_referer( 'chcounter-widget_update-options' );
			
			// Hidden order values are empty if Javascript is disabled
			if ($_POST['chcounter_widget_available_order'] == "" && $_POST['chcounter_widget_active_order'] == "") {
				$options['params']['available'] = $options['params']['active'] = $tmp = array();
				$param_order = $_POST['param_order'];
				asort($param_order); // sort parameter ordering keeping key associations intact
				foreach ($param_order AS $param => $order) {
					// Put fields with empty ordering values into available otherwise in active parameters 
					if ($order == "") {
						array_push($options['params']['available'], $param);
					} else {
						$order = intval($order);
						// prevent problems with identical keys
						if (array_key_exists($order, $tmp))
							array_push($tmp, $param);
						else
							$tmp[$order] = $param;
						ksort($tmp); // sort array by keys
						$options['params']['active'] = array_values($tmp);
					}
				}
			} else {
				// Parameter order from jQuery UI sortable list if Javascript is active
				$options['params']['available'] = $this->parseString($_POST['chcounter_widget_available_order']);
				$options['params']['active'] = $this->parseString($_POST['chcounter_widget_active_order']);
			}
			$options['invisible'] = isset( $_POST['chcounter_widget_invisible'] ) ? 1 : 0;
								
			update_option('chcounter_widget', $options);
			echo '<div id="message" class="updated fade"><p><strong>'.__( 'Settings saved', 'chcounter' ).'</strong></p></div>';
		}
		
		$options = get_option( 'chcounter_widget' );
		?>
		
		<div class='wrap' id="chcounter-widget-settings">
			<h2><?php _e( 'chCounter Widget Settings', 'chcounter' ) ?></h2>
			
			<p class="update-nag" id="chcounter_nojs"><?php _e('Javascript appears to be deactivated. You can activate chCounter parameters by inserting numbers giving their displaying order into the respective 	 fields. Empty values will deactivate parameters.', 'chcounter') ?></p>
			
			<form action='<?php esc_url(menu_page_url('chcounter-widget')) ?>' method='post' onSubmit="jQuery().setOrder();">
					
				<?php wp_nonce_field( 'chcounter-widget_update-options') ?>
				
				<div id="chcounter_available_box" class='chcounter_widget_parameters widget narrow' >
					<h3><?php _e( 'Available', 'chcounter' ) ?></h3>
					<ol class='chcounter_widget connectedSortable' id='chcounter_available'>
						<?php if ( count($options['params']['available']) > 0 ) : ?>
						<?php foreach ( $options['params']['available'] AS $order => $param ) : ?>
							<li class="widget-top ui-state-default" id='param_<?php echo $param ?>'><span class="widget-title"><?php echo $params[$param]['label'] ?></span><input type="number" min="0" step="1" name="param_order[<?php echo $param ?>]" class="param_order small-text" id="param_order_<?php echo $param ?>" size="2" value="" /></li>
						<?php endforeach; ?>
						<?php endif; ?>
					</ol>
					
					<span class="handle" id="chcounter_handle_available" style="display: none;"><?php _e( 'You see this message, because all parameters have been activated. To deactivate certain parameters simply drag & drop them into this box', 'chcounter' ) ?></span>
					<input type="hidden" name="chcounter_widget_available_order" id="chcounter_widget_available_order" value="" />
				</div>
				<div id="chcounter_active_box" class='chcounter_widget_parameters widget narrow'>
					<h3><?php _e( 'Active', 'chcounter' ) ?></h3>
							
					<ol class='chcounter_widget connectedSortable' id='chcounter_active'>
						<?php if ( count($options['params']['active']) > 0 ) : ?>
						<?php foreach ( $options['params']['active'] AS $order => $param ) : ?>
							<li class="widget-top ui-state-highlight" id='param_<?php echo $param ?>'><span class="widget-title"><?php echo $params[$param]['label'] ?></span><input type="number" min="0" step="1" name="param_order[<?php echo $param ?>]" class="param_order small-text" id="param_order_<?php echo $param ?>" size="2" value="<?php echo $order ?>" /></li>
						<?php endforeach; ?>
						<?php endif; ?>
					</ol>
					
					<span class="handle" id="chcounter_handle_active" style="display: none;"><?php _e( 'You see this message, because no parameters have been activated yet. You can create your chCounter Display via drag & drop into this box', 'chcounter' ) ?></span>					
					<input type="hidden" name="chcounter_widget_active_order" id="chcounter_widget_active_order" value="" />
				</div>
						
				<p class="submit"><input type="submit" name="updateSettings" value="<?php _e( 'Save Settings', 'chcounter' ) ?>" class="button button-primary" /></p>
			</form>
		</div>
		<?php
	}
	

	/**
	 * parse string and escape html specialchars
	 *
	 * @param string str
	 * @param string name
	 * @return escaped parsed string
	 */
	function parseString($str) {
		parse_str( $str, $tmp );
		return $tmp['param'];
		/*
		$ret = array();
		foreach ($tmp['param'] AS $id => $value) {
			$ret[] = htmlspecialchars($value);
		}
		return $ret;
		*/
	}
	
	
	/**
	 * save instance settings
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 * @return array
	 */
	function update( $new_instance, $old_instance )
	{
		$new_instance['title'] = htmlspecialchars($new_instance['title']);
		$new_instance['invisible'] = ( isset($new_instance['invisible']) && $new_instance['invisible'] == 1 ) ? 1 : 0;
		
		return $new_instance;
	}
	
	
	/**
	 * display control panel for the widget
	 *
	 * @param none
	 * @return void
	 */
	function form($instance)
	{
		if ( !isset($instance['title']) )
			$instance = array( 'title' => '', 'invisible' => 0 );
		
		$instance['invisible'] = ( isset($instance['invisible']) && $instance['invisible'] == 1 ) ? 1 : 0;
		echo '<p><label for="'.$this->get_field_id('title').'">'.__( 'Title', 'chcounter' ).'</label>: <input class="widefat" type="text" name="'.$this->get_field_name('title').'" id="'.$this->get_field_id('title').'" value="'.htmlspecialchars(stripslashes($instance['title'])).'" /></p>';
		echo '<p><label for="'.$this->get_field_id('invisible').'">'.__( 'Invisible', 'chcounter' ).'</label>&#160;<input type="checkbox" name="'.$this->get_field_name('invisible').'" id="'.$this->get_field_id('invisible').'"'.checked($instance['invisible'], 1, false).'" value="1" /></p>';
	}


	/**
	 * registers widget
	 *
	 * @param none
	 * @return void
	 */
	function register()
	{
		if ( !function_exists("register_sidebar_widget") )
			return;

		$widget_ops = array('classname' => 'widget_chcounter', 'description' => __('chCounter visitor statistics', 'chcounter') );
		wp_register_sidebar_widget( 'chcounter', 'chCounter', array(&$this, 'display'), $widget_ops );
		wp_register_widget_control( 'chcounter', 'chCounter', array(&$this, 'control'), array('width' => 250, 'height' => 100) );
	}
	
	
	/**
	 * Activate plugin
	 *
	 * @param none
	 * @return void
	 */
	function activate()
	{
		$params = $this->getParameters();

		$options = array();
		$options['version'] = $this->getVersion();
		$options['params'] = array( 'available' => array_keys($params), 'active' => array() );
		
		add_option( 'chcounter_widget', $options, '', 'yes' );
		
		/*
		* Add Capability to edit chCounter Widget Options for Administrator
		*/
		$role = get_role('administrator');
		$role->add_cap('edit_chcounter_widget');
		
		$this->writeChCounterConfig();
	}
	function writeChCounterConfig() {
		$content = "<?php
/*
 **************************************
 *
 * includes/config.inc.php
 * -------------
 *
 * last modified:	2005-04-21
 * -------------
 *
 * project:	chCounter
 * version:	3.1.1
 * copyright:	© 2005 Christoph Bachner
 * license:	GPL vs2.0 or higher [ see docs/license.txt ]
 * contact:	www.christoph-bachner.net
 *
 **************************************
*/


//
// Datenbank-Zugangsdaten
// Your database data 
// Données d'accès pour la base de données
//
\$_CHC_DBCONFIG = array(

	'server' => '".DB_HOST."',		// database server | Server | Server
	'user' => '".DB_USER."',			// database account | Benutzername | mot d'utilisateur
	'password' => '".DB_PASSWORD."',			// database password | Passwort | mot de passe
	'database' => '".DB_NAME."',			// database name | Datenbankname | nom de la base de données

	// Prefix of the chCounter database tables:
	// Präfix der chCounter Datenbanktabellen:
	// Préfixe des tableaux de la base de données du chCounter:
	'tables_prefix' => 'chc_'
	
);

?>";
		$file = $this->plugin_path.'/chcounter/includes/config.inc.php';
		file_put_contents($file, $content);
	}


	/**
	 * uninstall chCounter Widget
	 *
	 * @param none
	 * @return void
	 */
	function uninstall()
	{
		/*
		* Remove Capability to edit chCounter Widget Options for Administrator
		*/
		$role = get_role('administrator');
		$role->remove_cap('edit_chcounter_widget');
		
		delete_option( 'chcounter_widget' );
	}


	/**
	 * recursively remove directory
	 *
	 * @param string $dir
	 *
	 */
	function removeDir($dir)
	{
		$files = array_diff(scandir($dir), array('.','..'));
		foreach ($files AS $file) {
			if (is_dir("$dir/$file"))
				$this->removeDir("$dir/$file");
			else
				@unlink("$dir/$file");
		}
		@rmdir($dir);
	}
	
	
	/**
	 * check if chCounter is installed
	 *
	 * @param none
	 * @return boolean
	 */
	function chCounterIsInstalled()
	{
		global $wpdb;
		
		// check if table chc_config exists
		if ( $wpdb->query("SHOW TABLES LIKE 'chc_config'") == 1 )
			return true;
		
		return false;
	}
	
	
	/**
	 * add stylesheet
	 *
	 * @param none
	 * @return void
	 */
	function addStyles()
	{
		wp_enqueue_style('chcounter', $this->plugin_url."/style.css", array(), '1.0', 'all');
		
		wp_register_style('jquery-ui', $this->plugin_url . "/css/jquery/jquery-ui.min.css", false, '1.11.4', 'all');
		wp_register_style('jquery-ui-structure', $this->plugin_url . "/css/jquery/jquery-ui.structure.min.css", array('jquery-ui'), '1.11.4', 'all');
		wp_register_style('jquery-ui-theme', $this->plugin_url . "/css/jquery/jquery-ui.theme.min.css", array('jquery-ui', 'jquery-ui-structure'), '1.11.4', 'all');
		
		wp_enqueue_style('jquery-ui-structure');
		wp_enqueue_style('jquery-ui-theme');
		
		//wp_register_style('jquery_ui_css', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/themes/smoothness/jquery-ui.css', false, '1.0', 'screen');
		//wp_enqueue_style('jquery_ui_css');
	}

	
	/**
	 * add administration javascript
	 *
	 * @param none
	 * @return void
	 */
	function addScripts()
	{
		wp_enqueue_script('chcounter-scripts', $this->plugin_url.'/chcounter.js', array('jquery', 'jquery-ui-sortable'), '1.0');
	}
	

	/**
	 * addAdminMenu() - add options menu to the admin panel
	 *
	 * @param none
	 * @return void
	 */
	function addAdminMenu()
	{
		$plugin = basename(__FILE__,'.php').'/'.basename(__FILE__);
		$mypage = add_options_page( __( 'chCounter', 'chcounter' ), __( 'chCounter', 'chcounter' ), 'edit_chcounter_widget', 'chcounter-widget', array(&$this, 'displayAdminPage') );
		add_action( "admin_print_scripts-".$mypage, array(&$this, 'addScripts') );
		add_action( "admin_print_styles-".$mypage, array(&$this, 'addStyles') );
		add_filter( 'plugin_action_links_' . $plugin, array( &$this, 'pluginActions' ) );
	}
	
	
	/**
	 * display link to settings page in plugin table
	 *
	 * @param array $links array of action links
	 * @return new array of plugin actions
	 */
	function pluginActions( $links )
	{
		$settings_link = '<a href="'.esc_url(menu_page_url('chcounter-widget', 0)).'">' . __('Settings') . '</a>';
		array_unshift( $links, $settings_link );
	
		return $links;
	}
}

// run chCounter Widget
function chcounter_widget_init() {
	register_widget("chCounterWidget");
}
add_action('widgets_init', 'chcounter_widget_init');

// Initialize chCounter Widget
$chcounter_widget = new chCounterWidget( true );

/**
 * Wrapper function to display chCounter Widget statically
 *
 * @param string/array $args
 */
function chcounter_widget_display( $args = array() ) {
 	$chcounter_widget = new chCounterWidget();
	$chcounter_widget->widget( $args );
}
?>